@@ -1,9 +1,10 @@
 from typing import List
+import re
 
 from sqlalchemy import func, null
 
-from conda_store_server import orm
-from .conda import conda_platform
+from conda_store_server import orm, schema
+from conda_store_server.conda import conda_platform
 
 
 def list_namespaces(db, show_soft_deleted: bool = False):
@@ -24,8 +25,14 @@ def get_namespace(db, name: str = None, id: int = None):
 
 
 def create_namespace(db, name: str):
+    if re.fullmatch(f"[{schema.ALLOWED_CHARACTERS}]+", name) is None:
+        raise ValueError(
+            f"Namespace='{name}' is not valid does not match regex {schema.NAMESPACE_REGEX}"
+        )
+
     namespace = orm.Namespace(name=name)
     db.add(namespace)
+    return namespace
 
 
 def delete_namespace(db, name: str = None, id: int = None):
@@ -85,7 +92,7 @@ def get_specification(db, sha256):
 
 
 def post_specification(conda_store, specification, namespace=None):
-    conda_store.register_environment(specification, namespace)
+    return conda_store.register_environment(specification, namespace, force_build=True)
 
 
 def list_builds(db, status: orm.BuildStatus = None, show_soft_deleted: bool = False):
@@ -103,10 +110,15 @@ def get_build(db, build_id: int):
     return db.query(orm.Build).filter(orm.Build.id == build_id).first()
 
 
-def get_build_packages(db, build_id: int, search: str = None, build: str = None):
+def get_build_packages(
+    db, build_id: int, search: str = None, exact: bool = False, build: str = None
+):
     filters = [(orm.build_conda_package.c.build_id == build_id)]
     if search:
-        filters.append(orm.CondaPackage.name.contains(search, autoescape=True))
+        if exact:
+            filters.append(orm.CondaPackage.name.like(search.replace("%", r"\%")))
+        else:
+            filters.append(orm.CondaPackage.name.contains(search, autoescape=True))
     if build:
         filters.append(orm.CondaPackage.build.contains(build, autoescape=True))
 
@@ -178,10 +190,13 @@ def get_conda_channel(db, channel_name: str):
     )
 
 
-def list_conda_packages(db, search: str = None, build: str = None):
+def list_conda_packages(db, search: str = None, exact: bool = False, build: str = None):
     filters = []
     if search:
-        filters.append(orm.CondaPackage.name.contains(search, autoescape=True))
+        if exact:
+            filters.append(orm.CondaPackage.name.like(search.replace("%", r"\%")))
+        else:
+            filters.append(orm.CondaPackage.name.contains(search, autoescape=True))
     if build:
         filters.append(orm.CondaPackage.build.contains(build, autoescape=True))
 
