@@ -7,7 +7,7 @@ import subprocess
 import tempfile
 import traceback
 
-from sqlalchemy import and_
+import filelock
 import yaml
 
 from conda_store_server import api, conda, orm, utils, schema
@@ -34,12 +34,10 @@ def set_build_failed(conda_store, build, logs):
 
 
 def set_build_completed(conda_store, build, logs, packages):
-    unique_channel_urls = {p["base_url"] for p in packages}
-
-    channel_url_to_id_map = {}
-    for channel in unique_channel_urls:
+    for package in packages:
+        channel = package["channel_id"]
         if channel == "https://conda.anaconda.org/pypi":
-            # ignore pypi packages
+            # ignore pypi package for now
             continue
 
         channel_id = api.get_conda_channel(conda_store.db, channel)
@@ -47,34 +45,19 @@ def set_build_completed(conda_store, build, logs, packages):
             raise ValueError(
                 f"channel url={channel} not recognized in conda-store channel database"
             )
-        channel_url_to_id_map[channel] = channel_id.id
+        package["channel_id"] = channel_id.id
 
-    def package_query(package):
-        return (
+        _package = (
             conda_store.db.query(orm.CondaPackage)
-            .filter(
-                and_(
-                    orm.CondaPackage.channel_id
-                    == channel_url_to_id_map[package["base_url"]],
-                    orm.CondaPackage.subdir == package["platform"],
-                    orm.CondaPackage.name == package["name"],
-                    orm.CondaPackage.version == package["version"],
-                    orm.CondaPackage.build == package["build_string"],
-                    orm.CondaPackage.build_number == package["build_number"],
-                )
-            )
+            .filter(orm.CondaPackage.md5 == package["md5"])
+            .filter(orm.CondaPackage.channel_id == package["channel_id"])
             .first()
         )
 
-    for package in packages:
-        if package["base_url"] == "https://conda.anaconda.org/pypi":
-            # ignore pypi packages
-            continue
-
-        _package = package_query(package)
-
-        if _package is not None:
-            build.packages.append(_package)
+        if _package is None:
+            _package = orm.CondaPackage(**package)
+            conda_store.db.add(_package)
+        build.packages.append(_package)
 
     conda_store.storage.set(
         conda_store.db,
@@ -94,7 +77,7 @@ def set_build_completed(conda_store, build, logs, packages):
     directory_build_artifact = orm.BuildArtifact(
         build_id=build.id,
         artifact_type=orm.BuildArtifactType.DIRECTORY,
-        key=build.build_path(conda_store.store_directory),
+        key=build.build_path(conda_store),
     )
     conda_store.db.add(lockfile_build_artifact)
     conda_store.db.add(directory_build_artifact)
@@ -109,6 +92,22 @@ def set_build_completed(conda_store, build, logs, packages):
     conda_store.db.commit()
 
 
+def build_environment(conda_command, environment_filename, conda_prefix):
+    return subprocess.check_output(
+        [
+            conda_command,
+            "env",
+            "create",
+            "-p",
+            conda_prefix,
+            "-f",
+            str(environment_filename),
+        ],
+        stderr=subprocess.STDOUT,
+        encoding="utf-8",
+    )
+
+
 def build_conda_environment(conda_store, build):
     """Build a conda environment with set uid/gid/and permissions and
     symlink the build to a named environment
@@ -116,8 +115,11 @@ def build_conda_environment(conda_store, build):
     """
     set_build_started(conda_store, build)
 
-    conda_prefix = build.build_path(conda_store.store_directory)
-    environment_prefix = build.environment_path(conda_store.environment_directory)
+    conda_prefix = build.build_path(conda_store)
+    os.makedirs(os.path.dirname(conda_prefix), exist_ok=True)
+
+    environment_prefix = build.environment_path(conda_store)
+    os.makedirs(os.path.dirname(environment_prefix), exist_ok=True)
 
     conda_store.log.info(f"building conda environment={conda_prefix}")
 
@@ -127,19 +129,21 @@ def build_conda_environment(conda_store, build):
                 tmp_environment_filename = os.path.join(tmpdir, "environment.yaml")
                 with open(tmp_environment_filename, "w") as f:
                     yaml.dump(build.specification.spec, f)
-                    output = subprocess.check_output(
-                        [
+                    if conda_store.serialize_builds:
+                        with filelock.FileLock(
+                            os.path.join(tempfile.tempdir, "conda-store.lock")
+                        ):
+                            output = build_environment(
+                                conda_store.conda_command,
+                                tmp_environment_filename,
+                                conda_prefix,
+                            )
+                    else:
+                        output = build_environment(
                             conda_store.conda_command,
-                            "env",
-                            "create",
-                            "-p",
+                            tmp_environment_filename,
                             conda_prefix,
-                            "-f",
-                            str(tmp_environment_filename),
-                        ],
-                        stderr=subprocess.STDOUT,
-                        encoding="utf-8",
-                    )
+                        )
 
         utils.symlink(conda_prefix, environment_prefix)
 
@@ -169,7 +173,7 @@ def build_conda_environment(conda_store, build):
             with utils.timer(conda_store.log, f"chown of {conda_prefix}"):
                 utils.chown(conda_prefix, uid, gid)
 
-        packages = conda.conda_list(conda_prefix, executable=conda_store.conda_command)
+        packages = conda.conda_prefix_packages(conda_prefix)
         build.size = utils.disk_usage(conda_prefix)
 
         set_build_completed(conda_store, build, output.encode("utf-8"), packages)
@@ -184,7 +188,7 @@ def build_conda_environment(conda_store, build):
 
 
 def build_conda_env_export(conda_store, build):
-    conda_prefix = build.build_path(conda_store.store_directory)
+    conda_prefix = build.build_path(conda_store)
 
     output = subprocess.check_output(
         [conda_store.conda_command, "env", "export", "-p", conda_prefix]
@@ -205,7 +209,7 @@ def build_conda_env_export(conda_store, build):
 
 
 def build_conda_pack(conda_store, build):
-    conda_prefix = build.build_path(conda_store.store_directory)
+    conda_prefix = build.build_path(conda_store)
 
     conda_store.log.info(f"packaging archive of conda environment={conda_prefix}")
     with tempfile.TemporaryDirectory() as tmpdir:
@@ -230,7 +234,7 @@ def build_conda_docker(conda_store, build):
         fetch_precs,
     )
 
-    conda_prefix = build.build_path(conda_store.store_directory)
+    conda_prefix = build.build_path(conda_store)
 
     conda_store.log.info(f"creating docker archive of conda environment={conda_prefix}")
 
