@@ -23,7 +23,7 @@ from conda_store_server import schema, orm
 
 
 ARN_ALLOWED_REGEX = re.compile(
-    r"^([A-Za-z0-9|<>=\.\_\-\*]+)/([A-Za-z0-9|<>=\.\_\-\*]+)$"
+    f"^([{schema.ALLOWED_CHARACTERS}*]+)/([{schema.ALLOWED_CHARACTERS}*]+)$"
 )
 
 
@@ -123,9 +123,8 @@ class RBACAuthorizationBackend(LoggingConfigurable):
         if not ARN_ALLOWED_REGEX.match(arn):
             raise ValueError(f"invalid arn={arn}")
 
-        # replace "*" with "[A-Za-z0-9_\-\.|<>=]*"
-        arn = re.sub(r"\*", r"[A-Za-z0-9_\-\.|<>=]*", arn)
-
+        # replace "*" with schema.ALLOWED_CHARACTERS
+        arn = re.sub(r"\*", f"[{schema.ALLOWED_CHARACTERS}]*", arn)
         namespace_regex, name_regex = arn.split("/")
         regex_arn = "^" + namespace_regex + "(?:/" + name_regex + ")?$"
         return re.compile(regex_arn)
@@ -528,6 +527,7 @@ class GenericOAuthAuthentication(Authentication):
                 "grant_type": "authorization_code",
                 "client_id": self.client_id,
                 "client_secret": self.client_secret,
+                "redirect_uri": self.oauth_callback_url,
             },
             headers={"Accept": "application/json"},
             verify=self.tls_verify,
@@ -540,7 +540,7 @@ class GenericOAuthAuthentication(Authentication):
     def _get_username(self, authentication_token):
         response = requests.get(
             self.user_data_url,
-            headers={"Authorization": f"token {authentication_token}"},
+            headers={"Authorization": f"Bearer {authentication_token}"},
             verify=self.tls_verify,
         )
         response.raise_for_status()
