@@ -128,7 +128,7 @@ def api_get_namespace(namespace):
 
     namespace = api.get_namespace(conda_store.db, namespace)
     if namespace is None:
-        return jsonify({"status": "error", "error": "namespace does not exist"}), 404
+        return jsonify({"status": "error", "message": "namespace does not exist"}), 404
 
     return jsonify(
         {
@@ -147,11 +147,13 @@ def api_create_namespace(namespace):
 
     namespace_orm = api.get_namespace(conda_store.db, namespace)
     if namespace_orm:
-        return jsonify({"status": "error", "error": "namespace already exists"}), 409
+        return jsonify({"status": "error", "message": "namespace already exists"}), 409
 
-    api.create_namespace(conda_store.db, namespace)
+    try:
+        api.create_namespace(conda_store.db, namespace)
+    except ValueError as e:
+        return jsonify({"status": "error", "message": str(e.args[0])}), 400
     conda_store.db.commit()
-
     return jsonify({"status": "ok"})
 
 
@@ -164,7 +166,7 @@ def api_delete_namespace(namespace):
 
     namespace_orm = api.get_namespace(conda_store.db, namespace)
     if namespace_orm is None:
-        return jsonify({"status": "error", "error": "namespace does not exist"}), 404
+        return jsonify({"status": "error", "message": "namespace does not exist"}), 404
 
     conda_store.delete_namespace(namespace)
     return jsonify({"status": "ok"})
@@ -203,7 +205,10 @@ def api_get_environment(namespace, name):
 
     environment = api.get_environment(conda_store.db, namespace=namespace, name=name)
     if environment is None:
-        return jsonify({"status": "error", "error": "environment does not exist"}), 404
+        return (
+            jsonify({"status": "error", "message": "environment does not exist"}),
+            404,
+        )
 
     return jsonify(
         {
@@ -225,11 +230,11 @@ def api_update_environment_build(namespace, name):
     )
 
     data = request.json
-    if "buildId" not in data:
+    if "build_id" not in data:
         return jsonify({"status": "error", "message": "build id not specificated"}), 400
 
     try:
-        build_id = data["buildId"]
+        build_id = data["build_id"]
         conda_store.update_environment_build(namespace, name, build_id)
     except utils.CondaStoreError as e:
         return e.response
@@ -266,10 +271,13 @@ def api_post_specification():
         specification = request.json.get("specification")
         specification = yaml.safe_load(specification)
         specification = schema.CondaSpecification.parse_obj(specification)
-    except yaml.error.YAMLError as e:
-        return jsonify({"status": "error", "error": str(e)}), 400
+    except yaml.error.YAMLError:
+        return (
+            jsonify({"status": "error", "message": "Unable to parse. Invalid YAML"}),
+            400,
+        )
     except pydantic.ValidationError as e:
-        return jsonify({"status": "error", "error": e.errors()}), 400
+        return jsonify({"status": "error", "message": str(e)}), 400
 
     auth.authorize_request(
         f"{namespace_name}/{specification.name}",
@@ -277,8 +285,12 @@ def api_post_specification():
         require=True,
     )
 
-    api.post_specification(conda_store, specification, namespace_name)
-    return jsonify({"status": "ok"})
+    try:
+        build_id = api.post_specification(conda_store, specification, namespace_name)
+    except ValueError as e:
+        return jsonify({"status": "error", "message": str(e.args[0])}), 400
+
+    return jsonify({"status": "ok", "data": {"build_id": build_id}})
 
 
 @app_api.route("/api/v1/build/", methods=["GET"])
@@ -307,7 +319,7 @@ def api_get_build(build_id):
 
     build = api.get_build(conda_store.db, build_id)
     if build is None:
-        return jsonify({"status": "error", "error": "build id does not exist"}), 404
+        return jsonify({"status": "error", "message": "build id does not exist"}), 404
 
     auth.authorize_request(
         f"{build.environment.namespace.name}/{build.environment.name}",
@@ -330,11 +342,11 @@ def api_put_build(build_id):
 
     build = api.get_build(conda_store.db, build_id)
     if build is None:
-        return jsonify({"status": "error", "error": "build id does not exist"}), 404
+        return jsonify({"status": "error", "message": "build id does not exist"}), 404
 
     auth.authorize_request(
         f"{build.environment.namespace.name}/{build.environment.name}",
-        {Permissions.ENVIRONMENT_READ},
+        {Permissions.ENVIRONMENT_UPDATE},
         require=True,
     )
 
@@ -349,7 +361,7 @@ def api_delete_build(build_id):
 
     build = api.get_build(conda_store.db, build_id)
     if build is None:
-        return jsonify({"status": "error", "error": "build id does not exist"}), 404
+        return jsonify({"status": "error", "message": "build id does not exist"}), 404
 
     auth.authorize_request(
         f"{build.environment.namespace.name}/{build.environment.name}",
@@ -368,7 +380,7 @@ def api_get_build_packages(build_id):
 
     build = api.get_build(conda_store.db, build_id)
     if build is None:
-        return jsonify({"status": "error", "error": "build id does not exist"}), 404
+        return jsonify({"status": "error", "message": "build id does not exist"}), 404
 
     auth.authorize_request(
         f"{build.environment.namespace.name}/{build.environment.name}",
@@ -377,9 +389,11 @@ def api_get_build_packages(build_id):
     )
 
     search = request.args.get("search")
+    exact = request.args.get("exact")
+
     build_str = request.args.get("build")
     orm_packages = api.get_build_packages(
-        conda_store.db, build.id, search=search, build=build_str
+        conda_store.db, build.id, search=search, exact=exact, build=build_str
     )
     return paginated_api_response(
         orm_packages,
@@ -389,6 +403,7 @@ def api_get_build_packages(build_id):
             "name": orm.CondaPackage.name,
         },
         default_sort_by=["channel", "name"],
+        exclude={"channel": {"last_update"}},
     )
 
 
@@ -399,7 +414,7 @@ def api_get_build_logs(build_id):
 
     build = api.get_build(conda_store.db, build_id)
     if build is None:
-        return jsonify({"status": "error", "error": "build id does not exist"}), 404
+        return jsonify({"status": "error", "message": "build id does not exist"}), 404
 
     auth.authorize_request(
         f"{build.environment.namespace.name}/{build.environment.name}",
@@ -428,9 +443,13 @@ def api_list_packages():
     conda_store = get_conda_store()
 
     search = request.args.get("search")
+    exact = request.args.get("exact")
+
     build = request.args.get("build")
 
-    orm_packages = api.list_conda_packages(conda_store.db, search=search, build=build)
+    orm_packages = api.list_conda_packages(
+        conda_store.db, search=search, exact=exact, build=build
+    )
     required_sort_bys, distinct_orm_packages = filter_distinct_on(
         orm_packages,
         allowed_distinct_ons={
@@ -450,6 +469,7 @@ def api_list_packages():
         },
         default_sort_by=["channel", "name", "version", "build"],
         required_sort_bys=required_sort_bys,
+        exclude={"channel": {"last_update"}},
     )
 
 
@@ -460,7 +480,7 @@ def api_get_build_yaml(build_id):
 
     build = api.get_build(conda_store.db, build_id)
     if build is None:
-        return jsonify({"status": "error", "error": "build id does not exist"}), 404
+        return jsonify({"status": "error", "message": "build id does not exist"}), 404
 
     auth.authorize_request(
         f"{build.environment.namespace.name}/{build.environment.name}",
