@@ -1,15 +1,13 @@
 from flask import (
     Blueprint,
     render_template,
-    request,
     redirect,
     Response,
     url_for,
 )
-import pydantic
 import yaml
 
-from conda_store_server import api, schema
+from conda_store_server import api
 from conda_store_server.server.utils import get_conda_store, get_auth, get_server
 from conda_store_server.server.auth import Permissions
 from conda_store_server.conda import conda_platform
@@ -17,7 +15,7 @@ from conda_store_server.conda import conda_platform
 app_ui = Blueprint("ui", __name__, template_folder="templates")
 
 
-@app_ui.route("/create/", methods=["GET", "POST"])
+@app_ui.route("/create/", methods=["GET"])
 def ui_create_get_environment():
     conda_store = get_conda_store()
     auth = get_auth()
@@ -31,32 +29,7 @@ def ui_create_get_environment():
         "entity": auth.authenticate_request(),
     }
 
-    if request.method == "GET":
-        return render_template("create.html", **context)
-    elif request.method == "POST":
-        try:
-            namespace_id = int(request.form.get("namespace"))
-            specification_text = request.form.get("specification")
-            specification = schema.CondaSpecification.parse_obj(
-                yaml.safe_load(specification_text)
-            )
-            namespace = api.get_namespace(conda_store.db, id=namespace_id)
-            api.post_specification(conda_store, specification.dict(), namespace.name)
-            return redirect(url_for("ui.ui_list_environments"))
-        except yaml.YAMLError:
-            return render_template(
-                "create.html",
-                specification=specification_text,
-                message="Unable to parse. Invalid YAML",
-                **context,
-            )
-        except pydantic.ValidationError as e:
-            return render_template(
-                "create.html",
-                specification=specification_text,
-                message=str(e),
-                **context,
-            )
+    return render_template("create.html", **context)
 
 
 @app_ui.route("/", methods=["GET"])
